#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <sys/socket.h>
/*****These same functions are used for ttcp100, web100srv, and iperf100
   SO, if changes are made to web100api.c, 
   for iperf100:  
     copy the file over to the appropriate directory as 
       Iperf100subs.cpp 
     comment out: #include "web100.h" and #include "web100-int.h" 
     uncomment: #include "Iperf100.hpp" 
     set #define WEB100_LOGVARS to 0
     remake iperf100
   FOR ttcp100:
     copy the file over to the appropriate directory 
     comment out: #include "Iperf100.hpp"
     uncomment: #include "web100.h" and #include "web100-int.h" 
     set #define WEB100_LOGVARS to 0
     remake ttcp100
   FOR web100srv:
     copy the file over to the appropriate directory 
     comment out: #include "Iperf100.hpp"
     uncomment: #include "web100.h" and #include "web100-int.h" 
     set #define WEB100_LOGVARS equal to the number of items in the
       LogVar table.
     remake web100srv
*****/
#include "Iperf100.hpp" 
/*#include "web100.h"
#include "web100-int.h"
*/
/* files */
#define WEB100STATS "web100_variables"

/*Globals*/
/*******if NOT LOGGING, set WEB100_LOGVARS to 0!!**********/
#define WEB100_LOGVARS 0      /* > 0 ONLY for web100srv.c */
char LogVars[8][25]={"Timeouts", "CongestionSignals",
	"MinRTT", "MaxRwinSent", "MaxRwinRcvd", "PktsRetrans", 
	"SmoothedRTT", "PktsOut"};
/**********************************************************/
struct web100_var **stats100;
struct web100_agent *Agent;
int *mibvalues;
int VarCount=0, TotVars=0;
int NThreads=1, ThreadsLeft=0;
int Server=0;

/*Set up and initialize global web100 structures 
  web100_Init is called by ttcp100, iperf100 & web100srv
*/
struct web100_agent *web100_Init() {
  struct web100_agent *agent;
  struct web100_group *group;
  struct web100_var *var;
  char VarName[32], line[80];
  FILE *vf;
  int err, i=0, j=0;
  
/*Set up web100 structures per the web100 API*/
  if((agent = web100_attach(WEB100_AGENT_TYPE_LOCAL, NULL)) == NULL) {
    web100_perror("web100_attach");
    exit(EXIT_FAILURE);
  }
  Agent = agent;
/*Open and read user file to get web100 variables to display--
  - look up each variable to get a pointer to its web100_var struct
    - if the variable is not found, just don't include it
  - store the variable pointer in an array of pointers
*/
  if((vf = fopen(WEB100STATS, "r"))==NULL) {
    perror("web100_Init: web100_variables open error");
    exit(EXIT_FAILURE);
  }
/* Get a head count */
  while((fgets(line, 79, vf)) > 0) 
    VarCount++;
  rewind(vf);
  TotVars = VarCount + WEB100_LOGVARS;

/*Then malloc an array and look up each variable */   
  if((stats100 = (struct web100_var **) malloc(TotVars*sizeof(void *)))==NULL) {
    perror("web100_init malloc stats");
    exit(EXIT_FAILURE);
  }
  while((fscanf(vf, "%s", VarName))>0) {
      err=web100_agent_find_var_and_group(agent, VarName, &group, &var);
      if(!err) { 
	stats100[i]=var;
        i++;
      }
      else
	fprintf(stderr, "Variable %s not found!\n", VarName);
  }
  VarCount=i;
  fclose(vf);
  for(i=VarCount, j=0; j<WEB100_LOGVARS; j++) {
      err=web100_agent_find_var_and_group(agent, LogVars[j], &group, &var);
      if(!err) { 
	stats100[i]=var;
        i++;
      }
  }
  TotVars = i;
/*printf("VarCount: %d  TotVars: %d\n", VarCount, TotVars);
for(i=0; i<TotVars; i++) {
  printf("stats100[%d]->name: %s\n", i, stats100[i]->name);
}
*/
  return(agent);
}

struct web100_agent *web100_GetAgent() {
  return(Agent);
}

/* web100_Report is called by ttcp100, iperf100 & web100srv
*/
int *web100_Report(struct web100_connection *conn) {
  struct web100_group *fp;
  struct web100_snapshot *snap;
  int *tmpvalues;
  int j=0;
  unsigned int val;

/* malloc an array for values */
  if((tmpvalues = (int *) malloc(TotVars*sizeof(int)))==NULL) {
    perror("web100_Report malloc mibvalues");
    return(NULL);
  }

  fp = stats100[0]->group;
/*get a snapshot of the file containing the first variable*/
  if((snap = web100_snapshot_alloc(fp, conn))==NULL) {
    web100_perror("web100_snapshot_alloc");
    return(NULL);
  }
  if(web100_snap(snap)) {
    perror("web100_snap");
    if(web100_errno == WEB100_ERR_NOCONNECTION)
      printf("WEB100_ERR_NOCONNECTION\n");
    return(NULL);
  }

/*then read the values--getting snapshots of other files as needed--
    and store then in the array*/
  for(j=0; j<TotVars; j++) {
    if(stats100[j]->group != fp) {
      free(snap);
      fp = stats100[j]->group;
      if((snap = web100_snapshot_alloc(fp, conn))==NULL){
        web100_perror("web100_snapshot_alloc");
        return(NULL);
      }
      if(web100_snap(snap)) {
        perror("web100_snap");
        if(web100_errno == WEB100_ERR_NOCONNECTION) 
          printf("WEB100_ERR_NOCONNECTION\n");
        return(NULL);
      }
    }
    if(web100_snap_read(stats100[j], snap, &val)) {
      web100_perror("web100_snap_read");
      return(NULL);
    }
    tmpvalues[j] = val;
    if(tmpvalues[j] < 0)
      tmpvalues[j]=0;
/* printf("stats100[%d]->%s: %d--%u\n", j, stats100[j]->name, tmpvalues[j], val); */
  }
  free(snap);
  return((int *)tmpvalues);
}

/* web100_Print3 is called by ttcp100 & iperf100 */

int web100_Print3(int *tmpvalues, int sd) {
  int i=0, eol=0;
  char line[128]="", buff[64]="";

  for(i=0; i<VarCount; i++) {
    sprintf(buff, "%-13.13s %10u ", stats100[i]->name, tmpvalues[i]);
    strcat(line, buff);
    eol=0;
    if(!((i+1)%2)) {
      strcat(line, "\n");
      write(sd, line, strlen(line));
      line[0] = '\0';
      eol=1;
    }
  }
  if(!eol) {
      strcat(line, "\n");
      write(sd, line, strlen(line));
  }
  return(0);
}

/* web100_Print1 is called by web100srv */

int web100_Print1(int *tmpvalues, int sd) {
  int i=0;
  char line[128]="";

  for(i=0; i<TotVars; i++) {
    sprintf(line, "%s: %u\n", stats100[i]->name, tmpvalues[i]);
    write(sd, line, strlen(line));
  }
  return(0);
}

/*web100_ThreadCount is used in iperf100
*/
void web100_ThreadCount(int numthreads, int mode) {
/*Keep track of iperf threads..
  - When no threads are left, call web100_detach() 
*/
  if(mode == 1) {
    Server = 1;
  }
  else
    ThreadsLeft = NThreads = numthreads;
}

/* web100_ThreadCheck is used in iperf100 
*/
int web100_ThreadCheck(struct web100_agent *agent) {
  if(!Server) {
    ThreadsLeft--;
    if(ThreadsLeft == 0) {
      web100_detach(Agent);
      free(stats100);
    }
  }
  return(0);
}

/*web100_Logging is used in web100srv
*/
int web100_Logging(int *Timeouts,int *CongestionSignals,int *MinRTT,
  int *MaxRwinSent, int *MaxRwinRcvd, int *PktsRetrans, int *SmoothedRTT, int *PktsOut) {
  int j=VarCount;

  *Timeouts = mibvalues[j++];
  *CongestionSignals = mibvalues[j++];
  *MinRTT = mibvalues[j++];
  *MaxRwinSent = mibvalues[j++];
  *MaxRwinRcvd = mibvalues[j++];
  *PktsRetrans = mibvalues[j++];
  *SmoothedRTT = mibvalues[j++];
  *PktsOut = mibvalues[j];
  
  return(0);  
} 
